@@ -30,7 +30,8 @@
 #include "libglnx/libglnx.h"
 
 #include "flatpak-builtins.h"
-#include "flatpak-utils.h"
+#include "flatpak-utils-private.h"
+#include "parse-datetime.h"
 
 static char *opt_src_repo;
 static char *opt_src_ref;
@@ -39,21 +40,27 @@ static char *opt_body;
 static gboolean opt_update_appstream;
 static gboolean opt_no_update_summary;
 static gboolean opt_untrusted;
+static gboolean opt_disable_fsync;
 static gboolean opt_force;
 static char **opt_gpg_key_ids;
 static char *opt_gpg_homedir;
+static char *opt_endoflife;
+static char *opt_timestamp;
 
 static GOptionEntry options[] = {
   { "src-repo", 's', 0, G_OPTION_ARG_STRING, &opt_src_repo, N_("Source repo dir"), N_("SRC-REPO") },
-  { "src-ref", 's', 0, G_OPTION_ARG_STRING, &opt_src_ref, N_("Source repo ref"), N_("SRC-REF") },
+  { "src-ref", 0, 0, G_OPTION_ARG_STRING, &opt_src_ref, N_("Source repo ref"), N_("SRC-REF") },
   { "untrusted", 0, 0, G_OPTION_ARG_NONE, &opt_untrusted, "Do not trust SRC-REPO", NULL },
   { "force", 0, 0, G_OPTION_ARG_NONE, &opt_force, "Always commit, even if same content", NULL },
-  { "subject", 's', 0, G_OPTION_ARG_STRING, &opt_subject, N_("One line subject"), N_("SUBJECT") },
+  { "subject", 0, 0, G_OPTION_ARG_STRING, &opt_subject, N_("One line subject"), N_("SUBJECT") },
   { "body", 'b', 0, G_OPTION_ARG_STRING, &opt_body, N_("Full description"), N_("BODY") },
   { "update-appstream", 0, 0, G_OPTION_ARG_NONE, &opt_update_appstream, N_("Update the appstream branch"), NULL },
   { "no-update-summary", 0, 0, G_OPTION_ARG_NONE, &opt_no_update_summary, N_("Don't update the summary"), NULL },
   { "gpg-sign", 0, 0, G_OPTION_ARG_STRING_ARRAY, &opt_gpg_key_ids, N_("GPG Key ID to sign the commit with"), N_("KEY-ID") },
   { "gpg-homedir", 0, 0, G_OPTION_ARG_STRING, &opt_gpg_homedir, N_("GPG Homedir to use when looking for keyrings"), N_("HOMEDIR") },
+  { "end-of-life", 0, 0, G_OPTION_ARG_STRING, &opt_endoflife, N_("Mark build as end-of-life"), N_("REASON") },
+  { "timestamp", 0, 0, G_OPTION_ARG_STRING, &opt_timestamp, N_("Override the timestamp of the commit (NOW for current time)"), N_("TIMESTAMP") },
+  { "disable-fsync", 0, 0, G_OPTION_ARG_NONE, &opt_disable_fsync, "Do not invoke fsync()", NULL },
   { NULL }
 };
 
@@ -109,11 +116,12 @@ _ostree_get_relative_static_delta_path (const char *from,
 
 static GVariant *
 new_bytearray (const guchar *data,
-               gsize len)
+               gsize         len)
 {
   gpointer data_copy = g_memdup (data, len);
   GVariant *ret = g_variant_new_from_data (G_VARIANT_TYPE ("ay"), data_copy,
                                            len, FALSE, g_free, data_copy);
+
   return ret;
 }
 
@@ -122,9 +130,9 @@ rewrite_delta (OstreeRepo *src_repo,
                const char *src_commit,
                OstreeRepo *dst_repo,
                const char *dst_commit,
-               GVariant *dst_commitv,
+               GVariant   *dst_commitv,
                const char *from,
-               GError **error)
+               GError    **error)
 {
   g_autoptr(GFile) src_delta_file = NULL;
   g_autoptr(GFile) dst_delta_file = NULL;
@@ -167,12 +175,13 @@ rewrite_delta (OstreeRepo *src_repo,
 
   g_variant_dict_init (&dst_metadata_dict, src_metadata);
   g_variant_dict_remove (&dst_metadata_dict, src_detached_key);
-  if (ostree_repo_read_commit_detached_metadata (dst_repo, dst_commit, &dst_detached, NULL, NULL))
+  if (ostree_repo_read_commit_detached_metadata (dst_repo, dst_commit, &dst_detached, NULL, NULL) &&
+      dst_detached != NULL)
     g_variant_dict_insert_value (&dst_metadata_dict, dst_detached_key, dst_detached);
 
   g_variant_builder_add_value (&superblock_builder, g_variant_dict_end (&dst_metadata_dict));
   g_variant_builder_add_value (&superblock_builder, g_variant_get_child_value (src_superblock, 1)); /* timestamp */
-  g_variant_builder_add_value (&superblock_builder, from ? ostree_checksum_to_bytes_v (from) : new_bytearray ((guchar *)"", 0));
+  g_variant_builder_add_value (&superblock_builder, from ? ostree_checksum_to_bytes_v (from) : new_bytearray ((guchar *) "", 0));
   g_variant_builder_add_value (&superblock_builder, ostree_checksum_to_bytes_v (dst_commit));
   g_variant_builder_add_value (&superblock_builder, dst_commitv);
   g_variant_builder_add_value (&superblock_builder, src_recurse);
@@ -223,6 +232,8 @@ flatpak_builtin_build_commit_from (int argc, char **argv, GCancellable *cancella
   g_autoptr(GPtrArray) src_refs = NULL;
   g_autoptr(GPtrArray) resolved_src_refs = NULL;
   OstreeRepoCommitState src_commit_state;
+  struct timespec ts;
+  guint64 timestamp;
   int i;
 
   context = g_option_context_new (_("DST-REPO [DST-REF]... - Make a new commit based on existing commit(s)"));
@@ -236,7 +247,7 @@ flatpak_builtin_build_commit_from (int argc, char **argv, GCancellable *cancella
 
   dst_repo_arg = argv[1];
 
-  dst_refs = (const char **)argv + 2;
+  dst_refs = (const char **) argv + 2;
   n_dst_refs = argc - 2;
 
   if (opt_src_repo == NULL && n_dst_refs != 1)
@@ -248,6 +259,12 @@ flatpak_builtin_build_commit_from (int argc, char **argv, GCancellable *cancella
   if (opt_src_repo == NULL && opt_src_ref == NULL)
     return flatpak_fail (error, _("Either --src-repo or --src-ref must be specified."));
 
+  if (opt_timestamp)
+    {
+      if (!parse_datetime (&ts, opt_timestamp, NULL))
+        return flatpak_fail (error, _("Could not parse '%s'"), opt_timestamp);
+    }
+
   dst_repofile = g_file_new_for_commandline_arg (dst_repo_arg);
   if (!g_file_query_exists (dst_repofile, cancellable))
     return flatpak_fail (error, _("'%s' is not a valid repository"), dst_repo_arg);
@@ -256,6 +273,9 @@ flatpak_builtin_build_commit_from (int argc, char **argv, GCancellable *cancella
   if (!ostree_repo_open (dst_repo, cancellable, error))
     return FALSE;
 
+  if (opt_disable_fsync)
+    ostree_repo_set_disable_fsync (dst_repo, TRUE);
+
   if (opt_src_repo)
     {
       src_repofile = g_file_new_for_commandline_arg (opt_src_repo);
@@ -290,7 +310,7 @@ flatpak_builtin_build_commit_from (int argc, char **argv, GCancellable *cancella
                                       cancellable, error))
             return FALSE;
 
-          keys = (const char **)g_hash_table_get_keys_as_array (all_src_refs, NULL);
+          keys = (const char **) g_hash_table_get_keys_as_array (all_src_refs, NULL);
 
           for (i = 0; keys[i] != NULL; i++)
             {
@@ -299,7 +319,7 @@ flatpak_builtin_build_commit_from (int argc, char **argv, GCancellable *cancella
                 g_ptr_array_add (src_refs, g_strdup (keys[i]));
             }
           n_dst_refs = src_refs->len;
-          dst_refs = (const char **)src_refs->pdata;
+          dst_refs = (const char **) src_refs->pdata;
         }
       else
         {
@@ -326,6 +346,7 @@ flatpak_builtin_build_commit_from (int argc, char **argv, GCancellable *cancella
       GVariantBuilder builder;
       g_autoptr(OstreeAsyncProgress) progress = NULL;
       g_auto(GLnxConsoleRef) console = { 0, };
+      g_autoptr(GVariant) options = NULL;
       gboolean res;
 
       if (opt_untrusted)
@@ -339,13 +360,14 @@ flatpak_builtin_build_commit_from (int argc, char **argv, GCancellable *cancella
       g_variant_builder_add (&builder, "{s@v}", "flags",
                              g_variant_new_variant (g_variant_new_int32 (pullflags)));
       g_variant_builder_add (&builder, "{s@v}", "refs",
-                             g_variant_new_variant (g_variant_new_strv ((const char *const*) resolved_src_refs->pdata,
+                             g_variant_new_variant (g_variant_new_strv ((const char * const *) resolved_src_refs->pdata,
                                                                         resolved_src_refs->len)));
       g_variant_builder_add (&builder, "{s@v}", "depth",
                              g_variant_new_variant (g_variant_new_int32 (0)));
 
+      options = g_variant_ref_sink (g_variant_builder_end (&builder));
       res = ostree_repo_pull_with_options (dst_repo, src_repo_uri,
-                                           g_variant_builder_end (&builder),
+                                           options,
                                            progress,
                                            cancellable, error);
 
@@ -366,7 +388,7 @@ flatpak_builtin_build_commit_from (int argc, char **argv, GCancellable *cancella
   if (transaction == NULL)
     return FALSE;
 
-  for (i = 0; i < resolved_src_refs->len; i ++)
+  for (i = 0; i < resolved_src_refs->len; i++)
     {
       const char *dst_ref = dst_refs[i];
       const char *resolved_ref = g_ptr_array_index (resolved_src_refs, i);
@@ -378,6 +400,7 @@ flatpak_builtin_build_commit_from (int argc, char **argv, GCancellable *cancella
       g_autoptr(OstreeMutableTree) mtree = NULL;
       g_autoptr(GFile) dst_root = NULL;
       g_autoptr(GVariant) commitv_metadata = NULL;
+      g_autoptr(GVariant) metadata = NULL;
       const char *subject;
       const char *body;
       g_autofree char *commit_checksum = NULL;
@@ -418,23 +441,19 @@ flatpak_builtin_build_commit_from (int argc, char **argv, GCancellable *cancella
 
       commitv_metadata = g_variant_get_child_value (src_commitv, 0);
 
-      g_variant_get_child (src_commitv, 3, "s", &subject);
+      g_variant_get_child (src_commitv, 3, "&s", &subject);
       if (opt_subject)
-        subject = (const char *)opt_subject;
-      g_variant_get_child (src_commitv, 4, "s", &body);
+        subject = (const char *) opt_subject;
+      g_variant_get_child (src_commitv, 4, "&s", &body);
       if (opt_body)
-        body = (const char *)opt_body;
+        body = (const char *) opt_body;
 
-#ifdef FLATPAK_ENABLE_P2P
       dst_collection_id = ostree_repo_get_collection_id (dst_repo);
-#endif  /* FLATPAK_ENABLE_P2P */
 
       /* Copy old metadata */
       g_variant_builder_init (&metadata_builder, G_VARIANT_TYPE ("a{sv}"));
 
-      /* Bindings. xa.ref is deprecated but added anyway for backwards compatibility.
-       * Add the bindings even if we are not built with P2P support, since other
-       * flatpak builds might be. */
+      /* Bindings. xa.ref is deprecated but added anyway for backwards compatibility. */
       g_variant_builder_add (&metadata_builder, "{sv}", "ostree.collection-binding",
                              g_variant_new_string (dst_collection_id ? dst_collection_id : ""));
       g_variant_builder_add (&metadata_builder, "{sv}", "ostree.ref-binding",
@@ -459,12 +478,25 @@ flatpak_builtin_build_commit_from (int argc, char **argv, GCancellable *cancella
               strcmp (key, "ostree.ref-binding") == 0)
             continue;
 
+          if (opt_endoflife &&
+              strcmp (key, OSTREE_COMMIT_META_KEY_ENDOFLIFE) == 0)
+            continue;
+
           g_variant_builder_add_value (&metadata_builder, child);
         }
 
-      if (!ostree_repo_write_commit_with_time (dst_repo, dst_parent, subject, body, g_variant_builder_end (&metadata_builder),
+      if (opt_endoflife && *opt_endoflife)
+        g_variant_builder_add (&metadata_builder, "{sv}", OSTREE_COMMIT_META_KEY_ENDOFLIFE,
+                               g_variant_new_string (opt_endoflife));
+
+      timestamp = ostree_commit_get_timestamp (src_commitv);
+      if (opt_timestamp)
+        timestamp = ts.tv_sec;
+
+      metadata = g_variant_ref_sink (g_variant_builder_end (&metadata_builder));
+      if (!ostree_repo_write_commit_with_time (dst_repo, dst_parent, subject, body, metadata,
                                                OSTREE_REPO_FILE (dst_root),
-                                               ostree_commit_get_timestamp (src_commitv),
+                                               timestamp,
                                                &commit_checksum, cancellable, error))
         return FALSE;
 
@@ -500,14 +532,12 @@ flatpak_builtin_build_commit_from (int argc, char **argv, GCancellable *cancella
             }
         }
 
-#ifdef FLATPAK_ENABLE_P2P
       if (dst_collection_id != NULL)
         {
           OstreeCollectionRef ref = { (char *) dst_collection_id, (char *) dst_ref };
           ostree_repo_transaction_set_collection_ref (dst_repo, &ref, commit_checksum);
         }
       else
-#endif  /* FLATPAK_ENABLE_P2P */
         {
           ostree_repo_transaction_set_ref (dst_repo, NULL, dst_ref, commit_checksum);
         }
