@@ -29,13 +29,13 @@
 #include <glib/gi18n.h>
 
 #include "libglnx/libglnx.h"
-#include "flatpak-document-dbus.h"
+#include "flatpak-document-dbus-generated.h"
 
 #include <gio/gunixfdlist.h>
 
 #include "flatpak-builtins.h"
-#include "flatpak-utils.h"
-#include "flatpak-run.h"
+#include "flatpak-utils-private.h"
+#include "flatpak-run-private.h"
 
 static gboolean opt_unique = FALSE;
 static gboolean opt_transient = FALSE;
