@@ -30,9 +30,9 @@
 #include "libglnx/libglnx.h"
 
 #include "flatpak-builtins.h"
-#include "flatpak-utils.h"
+#include "flatpak-utils-private.h"
 #include "flatpak-builtins-utils.h"
-#include "flatpak-run.h"
+#include "flatpak-run-private.h"
 
 static gboolean opt_user;
 static gboolean opt_system;
@@ -41,8 +41,11 @@ static gboolean opt_show_commit;
 static gboolean opt_show_origin;
 static gboolean opt_show_size;
 static gboolean opt_show_metadata;
+static gboolean opt_show_runtime;
+static gboolean opt_show_sdk;
 static gboolean opt_show_permissions;
 static gboolean opt_show_extensions;
+static gboolean opt_show_location;
 static char *opt_arch;
 static char **opt_installations;
 static char *opt_file_access;
@@ -57,9 +60,12 @@ static GOptionEntry options[] = {
   { "show-origin", 'o', 0, G_OPTION_ARG_NONE, &opt_show_origin, N_("Show origin"), NULL },
   { "show-size", 's', 0, G_OPTION_ARG_NONE, &opt_show_size, N_("Show size"), NULL },
   { "show-metadata", 'm', 0, G_OPTION_ARG_NONE, &opt_show_metadata, N_("Show metadata"), NULL },
+  { "show-runtime", 0, 0, G_OPTION_ARG_NONE, &opt_show_runtime, N_("Show runtime"), NULL },
+  { "show-sdk", 0, 0, G_OPTION_ARG_NONE, &opt_show_sdk, N_("Show sdk"), NULL },
   { "show-permissions", 'M', 0, G_OPTION_ARG_NONE, &opt_show_permissions, N_("Show permissions"), NULL },
   { "file-access", 0, 0, G_OPTION_ARG_FILENAME, &opt_file_access, N_("Query file access"), N_("PATH") },
   { "show-extensions", 'e', 0, G_OPTION_ARG_NONE, &opt_show_extensions, N_("Show extensions"), NULL },
+  { "show-location", 'l', 0, G_OPTION_ARG_NONE, &opt_show_location, N_("Show location"), NULL },
   { NULL }
 };
 
@@ -74,7 +80,7 @@ maybe_print_space (gboolean *first)
 }
 
 static gchar *
-format_timestamp (guint64  timestamp)
+format_timestamp (guint64 timestamp)
 {
   GDateTime *dt;
   gchar *str;
@@ -94,12 +100,15 @@ flatpak_builtin_info (int argc, char **argv, GCancellable *cancellable, GError *
 {
   g_autoptr(GOptionContext) context = NULL;
   g_autofree char *ref = NULL;
-  FlatpakDir *dir = NULL;
+  g_autoptr(FlatpakDir) dir = NULL;
   g_autoptr(GVariant) deploy_data = NULL;
   g_autoptr(FlatpakDeploy) deploy = NULL;
+  g_autoptr(GFile) deploy_dir = NULL;
   g_autoptr(GKeyFile) metakey = NULL;
   const char *commit = NULL;
   const char *alt_id = NULL;
+  const char *eol;
+  const char *eol_rebase;
   const char *pref = NULL;
   const char *default_branch = NULL;
   const char *origin = NULL;
@@ -115,7 +124,7 @@ flatpak_builtin_info (int argc, char **argv, GCancellable *cancellable, GError *
   gboolean friendly = TRUE;
   g_autofree const char **subpaths = NULL;
 
-  context = g_option_context_new (_("NAME [BRANCH] - Get info about installed app and/or runtime"));
+  context = g_option_context_new (_("NAME [BRANCH] - Get info about an installed app or runtime"));
   g_option_context_set_translation_domain (context, GETTEXT_PACKAGE);
 
   if (!flatpak_option_context_parse (context, options, &argc, &argv, FLATPAK_BUILTIN_FLAG_NO_DIR, NULL, cancellable, error))
@@ -146,7 +155,7 @@ flatpak_builtin_info (int argc, char **argv, GCancellable *cancellable, GError *
   if (deploy_data == NULL)
     return FALSE;
 
-  deploy = flatpak_find_deploy_for_ref (ref, NULL, error);
+  deploy = flatpak_find_deploy_for_ref (ref, NULL, NULL, error);
   if (deploy == NULL)
     return FALSE;
 
@@ -163,12 +172,16 @@ flatpak_builtin_info (int argc, char **argv, GCancellable *cancellable, GError *
   origin = flatpak_deploy_data_get_origin (deploy_data);
   size = flatpak_deploy_data_get_installed_size (deploy_data);
   formatted = g_format_size (size);
-  path = g_file_get_path (flatpak_deploy_get_dir (deploy));
+  deploy_dir = flatpak_deploy_get_dir (deploy);
+  path = g_file_get_path (deploy_dir);
   subpaths = flatpak_deploy_data_get_subpaths (deploy_data);
+  eol = flatpak_deploy_data_get_eol (deploy_data);
+  eol_rebase = flatpak_deploy_data_get_eol_rebase (deploy_data);
 
   metakey = flatpak_deploy_get_metadata (deploy);
 
-  if (opt_show_ref || opt_show_origin || opt_show_commit || opt_show_size || opt_show_metadata || opt_show_permissions || opt_file_access)
+  if (opt_show_ref || opt_show_origin || opt_show_commit || opt_show_size || opt_show_metadata || opt_show_permissions ||
+      opt_file_access || opt_show_location || opt_show_runtime || opt_show_sdk)
     friendly = FALSE;
 
   if (friendly)
@@ -180,13 +193,13 @@ flatpak_builtin_info (int argc, char **argv, GCancellable *cancellable, GError *
       const gchar *subject = NULL;
       const gchar *body = NULL;
       g_autofree char *parent = NULL;
-      const char *latest;
+      g_autofree char *latest = NULL;
       const char *xa_metadata = NULL;
       const char *collection_id = NULL;
 
       latest = flatpak_dir_read_latest (dir, origin, ref, NULL, NULL, NULL);
       if (latest == NULL)
-        latest = _("ref not present in origin");
+        latest = g_strdup (_("ref not present in origin"));
 
       if (ostree_repo_load_commit (flatpak_dir_get_repo (dir), commit, &commit_v, NULL, NULL))
         {
@@ -201,9 +214,7 @@ flatpak_builtin_info (int argc, char **argv, GCancellable *cancellable, GError *
           if (xa_metadata == NULL)
             g_printerr (_("Warning: Commit has no flatpak metadata\n"));
 
-#ifdef FLATPAK_ENABLE_P2P
           g_variant_lookup (commit_metadata, "ostree.collection-binding", "&s", &collection_id);
-#endif
         }
 
       g_print ("%s%s%s %s\n", on, _("Ref:"), off, ref);
@@ -230,11 +241,18 @@ flatpak_builtin_info (int argc, char **argv, GCancellable *cancellable, GError *
       g_print ("%s%s%s %s\n", on, _("Parent:"), off, parent ? parent : "-");
       g_print ("%s%s%s %s\n", on, _("Location:"), off, path);
       g_print ("%s%s%s %s\n", on, _("Installed size:"), off, formatted);
+      if (eol)
+        g_print ("%s%s%s %s\n", on, _("end-of-life:"), off, eol);
+      if (eol_rebase)
+        g_print ("%s%s%s %s\n", on, _("end-of-life-rebase:"), off, eol_rebase);
       if (strcmp (parts[0], "app") == 0)
         {
           g_autofree char *runtime = NULL;
+          g_autofree char *sdk = NULL;
           runtime = g_key_file_get_string (metakey, "Application", "runtime", error);
           g_print ("%s%s%s %s\n", on, _("Runtime:"), off, runtime ? runtime : "-");
+          sdk = g_key_file_get_string (metakey, "Application", "sdk", error);
+          g_print ("%s%s%s %s\n", on, _("Sdk:"), off, sdk ? sdk : "-");
         }
       if (subpaths[0] != NULL)
         {
@@ -271,6 +289,36 @@ flatpak_builtin_info (int argc, char **argv, GCancellable *cancellable, GError *
           g_print ("%s", formatted);
         }
 
+      if (opt_show_location)
+        {
+          maybe_print_space (&first);
+          g_print ("%s", path);
+        }
+
+      if (opt_show_runtime)
+        {
+          g_autofree char *runtime = NULL;
+          maybe_print_space (&first);
+
+          if (strcmp (parts[0], "app") == 0)
+            runtime = g_key_file_get_string (metakey, "Application", "runtime", NULL);
+          else
+            runtime = g_key_file_get_string (metakey, "Runtime", "runtime", NULL);
+          g_print ("%s", runtime ? runtime : "-");
+        }
+
+      if (opt_show_sdk)
+        {
+          g_autofree char *sdk = NULL;
+          maybe_print_space (&first);
+
+          if (strcmp (parts[0], "app") == 0)
+            sdk = g_key_file_get_string (metakey, "Application", "sdk", NULL);
+          else
+            sdk = g_key_file_get_string (metakey, "Runtime", "sdk", NULL);
+          g_print ("%s", sdk ? sdk : "-");
+        }
+
       if (!first)
         g_print ("\n");
 
@@ -329,51 +377,51 @@ flatpak_builtin_info (int argc, char **argv, GCancellable *cancellable, GError *
 
   if (opt_show_extensions)
     {
-       GList *extensions, *l;
+      GList *extensions, *l;
 
-       extensions = flatpak_list_extensions (metakey, parts[2], parts[3]);
-       for (l = extensions; l; l = l->next)
-         {
-           FlatpakExtension *ext = l->data;
-           g_autofree const char **subpaths = NULL;
-           g_autoptr(GVariant) ext_deploy_data = NULL;
-           g_autofree char *formatted = NULL;
+      extensions = flatpak_list_extensions (metakey, parts[2], parts[3]);
+      for (l = extensions; l; l = l->next)
+        {
+          FlatpakExtension *ext = l->data;
+          g_autofree const char **subpaths = NULL;
+          g_autoptr(GVariant) ext_deploy_data = NULL;
+          g_autofree char *formatted = NULL;
 
-           if (ext->is_unmaintained)
-             {
-               commit = "unmaintained";
-               origin = NULL;
-               size = 0;
-               formatted = g_strdup ("unknown");
-               subpaths = NULL;
-             }
-           else
-             {
-               ext_deploy_data = flatpak_dir_get_deploy_data (dir, ext->ref, cancellable, error);
-               if (ext_deploy_data == NULL)
-                 return FALSE;
+          if (ext->is_unmaintained)
+            {
+              commit = "unmaintained";
+              origin = NULL;
+              size = 0;
+              formatted = g_strdup ("unknown");
+              subpaths = NULL;
+            }
+          else
+            {
+              ext_deploy_data = flatpak_dir_get_deploy_data (dir, ext->ref, cancellable, error);
+              if (ext_deploy_data == NULL)
+                return FALSE;
 
-               commit = flatpak_deploy_data_get_commit (ext_deploy_data);
-               origin = flatpak_deploy_data_get_origin (ext_deploy_data);
-               size = flatpak_deploy_data_get_installed_size (ext_deploy_data);
-               formatted = g_format_size (size);
-               subpaths = flatpak_deploy_data_get_subpaths (ext_deploy_data);
-             }
+              commit = flatpak_deploy_data_get_commit (ext_deploy_data);
+              origin = flatpak_deploy_data_get_origin (ext_deploy_data);
+              size = flatpak_deploy_data_get_installed_size (ext_deploy_data);
+              formatted = g_format_size (size);
+              subpaths = flatpak_deploy_data_get_subpaths (ext_deploy_data);
+            }
 
-           g_print ("\n%s%s%s %s\n", on, _("Extension:"), off, ext->ref);
-           g_print ("%s%s%s %s\n", on, _("ID:"), off, ext->id);
-           g_print ("%s%s%s %s\n", on, _("Origin:"), off, origin ? origin : "-");
-           g_print ("%s%s%s %s\n", on, _("Commit:"), off, commit);
-           g_print ("%s%s%s %s%s\n", on, _("Installed size:"), off, subpaths && subpaths[0] ? "<" : "", formatted);
+          g_print ("\n%s%s%s %s\n", on, _("Extension:"), off, ext->ref);
+          g_print ("%s%s%s %s\n", on, _("ID:"), off, ext->id);
+          g_print ("%s%s%s %s\n", on, _("Origin:"), off, origin ? origin : "-");
+          g_print ("%s%s%s %s\n", on, _("Commit:"), off, commit);
+          g_print ("%s%s%s %s%s\n", on, _("Installed size:"), off, subpaths && subpaths[0] ? "<" : "", formatted);
 
-           if (subpaths && subpaths[0])
-             {
-               g_autofree char *subpath_str = NULL;
+          if (subpaths && subpaths[0])
+            {
+              g_autofree char *subpath_str = NULL;
 
-               subpath_str = g_strjoinv (",", (char **)subpaths);
-               g_print ("%s%s%s %s\n", on, _("Subpaths:"), off, subpath_str);
-             }
-         }
+              subpath_str = g_strjoinv (",", (char **) subpaths);
+              g_print ("%s%s%s %s\n", on, _("Subpaths:"), off, subpath_str);
+            }
+        }
     }
 
   return TRUE;
@@ -390,7 +438,7 @@ flatpak_complete_info (FlatpakCompletion *completion)
 
   context = g_option_context_new ("");
   if (!flatpak_option_context_parse (context, options, &completion->argc, &completion->argv,
-                                     FLATPAK_BUILTIN_FLAG_ALL_DIRS, &dirs, NULL, NULL))
+                                     FLATPAK_BUILTIN_FLAG_ALL_DIRS | FLATPAK_BUILTIN_FLAG_OPTIONAL_REPO, &dirs, NULL, NULL))
     return FALSE;
 
   kinds = FLATPAK_KINDS_APP | FLATPAK_KINDS_RUNTIME;
