@@ -30,7 +30,7 @@
 #include "libglnx/libglnx.h"
 
 #include "flatpak-builtins.h"
-#include "flatpak-utils.h"
+#include "flatpak-utils-private.h"
 #include "flatpak-table-printer.h"
 
 static gboolean opt_show_details;
@@ -53,14 +53,15 @@ static GOptionEntry options[] = {
 typedef struct
 {
   FlatpakDir *dir;
-  GStrv app_refs;
-  GStrv runtime_refs;
+  GStrv       app_refs;
+  GStrv       runtime_refs;
 } RefsData;
 
 static RefsData *
 refs_data_new (FlatpakDir *dir, const GStrv app_refs, const GStrv runtime_refs)
 {
   RefsData *refs_data = g_new0 (RefsData, 1);
+
   refs_data->dir = g_object_ref (dir);
   refs_data->app_refs = g_strdupv ((char **) app_refs);
   refs_data->runtime_refs = g_strdupv ((char **) runtime_refs);
@@ -116,7 +117,7 @@ find_refs_for_dir (FlatpakDir *dir, GStrv *apps, GStrv *runtimes, GCancellable *
 }
 
 static gboolean
-print_table_for_refs (gboolean print_apps, GPtrArray* refs_array, const char *arch, GCancellable *cancellable, GError **error)
+print_table_for_refs (gboolean print_apps, GPtrArray * refs_array, const char *arch, GCancellable *cancellable, GError **error)
 {
   FlatpakTablePrinter *printer = flatpak_table_printer_new ();
   int i;
@@ -159,6 +160,8 @@ print_table_for_refs (gboolean print_apps, GPtrArray* refs_array, const char *ar
           g_autoptr(GVariant) deploy_data = NULL;
           const char *active;
           const char *alt_id;
+          const char *eol;
+          const char *eol_rebase;
           g_autofree char *latest = NULL;
           g_autofree char *size_s = NULL;
           guint64 size = 0;
@@ -195,6 +198,8 @@ print_table_for_refs (gboolean print_apps, GPtrArray* refs_array, const char *ar
 
           active = flatpak_deploy_data_get_commit (deploy_data);
           alt_id = flatpak_deploy_data_get_alt_id (deploy_data);
+          eol = flatpak_deploy_data_get_eol (deploy_data);
+          eol_rebase = flatpak_deploy_data_get_eol_rebase (deploy_data);
 
           latest = flatpak_dir_read_latest (dir, repo, ref, NULL, NULL, NULL);
           if (latest)
@@ -237,10 +242,7 @@ print_table_for_refs (gboolean print_apps, GPtrArray* refs_array, const char *ar
             }
 
           if (alt_id)
-            {
-              g_autofree char *alt_id_str = g_strdup_printf ("alt-id=%.12s", alt_id);
-              flatpak_table_printer_append_with_comma (printer, alt_id_str);
-            }
+            flatpak_table_printer_append_with_comma_printf (printer, "alt-id=%.12s", alt_id);
 
           if (strcmp (parts[0], "app") == 0)
             {
@@ -261,6 +263,12 @@ print_table_for_refs (gboolean print_apps, GPtrArray* refs_array, const char *ar
             {
               flatpak_table_printer_append_with_comma (printer, "partial");
             }
+
+          if (eol)
+            flatpak_table_printer_append_with_comma_printf (printer, "eol=%s", eol);
+          if (eol_rebase)
+            flatpak_table_printer_append_with_comma_printf (printer, "eol-rebase=%s", eol_rebase);
+
           flatpak_table_printer_finish_row (printer);
         }
     }
@@ -306,7 +314,7 @@ flatpak_builtin_list (int argc, char **argv, GCancellable *cancellable, GError *
   g_option_context_set_translation_domain (context, GETTEXT_PACKAGE);
 
   if (!flatpak_option_context_parse (context, options, &argc, &argv,
-                                     FLATPAK_BUILTIN_FLAG_ALL_DIRS,
+                                     FLATPAK_BUILTIN_FLAG_ALL_DIRS | FLATPAK_BUILTIN_FLAG_OPTIONAL_REPO,
                                      &dirs, cancellable, error))
     return FALSE;
 
