@@ -49,6 +49,7 @@ GType flatpak_deploy_get_type (void);
 #define FLATPAK_REF_NAME_KEY "Name"
 #define FLATPAK_REF_BRANCH_KEY "Branch"
 #define FLATPAK_REF_COLLECTION_ID_KEY "CollectionID"
+#define FLATPAK_REF_DEPLOY_COLLECTION_ID_KEY "DeployCollectionID"
 
 #define FLATPAK_REPO_GROUP "Flatpak Repo"
 #define FLATPAK_REPO_VERSION_KEY "Version"
@@ -59,6 +60,7 @@ GType flatpak_deploy_get_type (void);
 #define FLATPAK_REPO_NODEPS_KEY "NoDeps"
 
 #define FLATPAK_REPO_COLLECTION_ID_KEY "CollectionID"
+#define FLATPAK_REPO_DEPLOY_COLLECTION_ID_KEY "DeployCollectionID"
 
 #define FLATPAK_DEFAULT_UPDATE_FREQUENCY 100
 #define FLATPAK_CLI_UPDATE_FREQUENCY 300
